-- Policies grant buildings
-- Author: Machiavelli
-- DateCreated: 6/27/2014 7:46:43 PM
--------------------------------------------------------------
-- Version 1
--
-- Purpose:
-- When a player adopts a policy, founds a city or captures a city
-- LuaEvents.GetPolicyBuildings gets called for the city (or every city
-- if a policy was adopted).  Tables (like those in PGB_Tables.xml) hook
-- into LuaEvents.GetPolicyBuildings and add their valid buildings to the
-- buildingList.  When this code gets the populated buildingList back it
-- adds those buildings to the city.
--------------------------------------------------------------

------------------------------
-- Private helper functions --
------------------------------
function GetBuildingTypeFromClass(buildingClass, civilizationTypeID)
	-- Assume it is the default building
	local buildingType = GameInfo.BuildingClasses[buildingClass].DefaultBuilding;

	-- See if this civilization has a unique building for this building class
	local condition = "CivilizationType = '" .. GameInfo.Civilizations[civilizationTypeID].Type .. "' AND BuildingClassType = '" .. buildingClass .. "'";
	for row in GameInfo.Civilization_BuildingClassOverrides(condition) do
		buildingType = row.BuildingType;
		return buildingType;
	end

	return buildingType;
end

----------------------------------
-- functions that add buildings --
----------------------------------
-- Used as a wrapper for AddPolicyBuildingsToCity
function PolicyBuildings_CityCaptured(oldPlayerID, bCapital, iX, iY, newPlayerID, conquest, conquest2)
	local player = Players[newPlayerID];

	if(not player:IsMinorCiv() and not player:IsAlien()) then
		AddPolicyBuildingsToCity(newPlayerID, Map.GetPlot(iX, iY):GetPlotCity():GetID());
	end
end
--GameEvents.CityCaptureComplete.Add(PolicyBuildings_CityCaptured);

-- Used as a wrapper for AddPolicyBuildingsToCity
function PolicyBuildings_CityFounded(playerID, iX, iY)
	local player = Players[playerID];

	if(not player:IsMinorCiv() and not player:IsAlien()) then
		AddPolicyBuildingsToCity(playerID, Map.GetPlot(iX, iY):GetPlotCity():GetID());
	end
end
--GameEvents.PlayerCityFounded.Add(PolicyBuildings_CityFounded);

function PolicyBuildings_PolicyAdopted(playerID, policyID)
	for city in Players[playerID]:Cities() do
		AddPolicyBuildingsToCity(playerID, city:GetID());
	end
end
--GameEvents.PlayerAdoptPolicy.Add(PolicyBuildings_PolicyAdopted);

function PolicyBuildings_PolicyBranchAdopted(playerID, policyBranchID)
	local player = Players[playerID];

	local freePolicy = GameInfo.PolicyBranchTypes[policyBranchID].FreePolicy;
	local policyID = GameInfo.Policies[freePolicy].ID;

	PolicyBuildings_PolicyAdopted(playerID, policyID);
end
--GameEvents.PlayerAdoptPolicyBranch.Add(PolicyBuildings_PolicyBranchAdopted);

function AddPolicyBuildingsToCity(playerID, cityID)
	local player = Players[playerID];
	local city = player:GetCityByID(cityID);
	local buildingList = {}; -- A list of buildingClass strings to add to the city
	local buildingType;

	-- All XML tables should hook into this event and add building class strings to the buildingList table.
	LuaEvents.GetPolicyBuildings(playerID, cityID, buildingList);

	for k, v in pairs(buildingList) do
		buildingType = GetBuildingTypeFromClass(k, player:GetCivilizationType());

		if(buildingType ~= nil and buildingType ~= "") then
			city:SetNumRealBuilding(GameInfo.Buildings[buildingType].ID, 1);
		end
	end
end

--------------
-- Initialization check.  Ensures this code isn't loaded twice
--------------
local retVal = {};
local versionNumber = 2;
function isInitialized(retVal)
	retVal.isInitialized = versionNumber;
end
LuaEvents.PoliciesGrantBuildings_IsInitialized(retVal);

-- If an older version has been initialized by another mod, remove it
if(retVal.isInitialized ~= nil and retVal.isInitialized < versionNumber) then
	LuaEvents.PoliciesGrantBuildings_IsInitialized.Remove(isInitialized);
	-- Remove the old code
	GameEvents.CityCaptureComplete.Remove(PolicyBuildings_CityCaptured);
	GameEvents.PlayerCityFounded.Remove(PolicyBuildings_CityFounded);
	GameEvents.PlayerAdoptPolicy.Remove(PolicyBuildings_PolicyAdopted);
	GameEvents.PlayerAdoptPolicyBranch.Remove(PolicyBuildings_PolicyBranchAdopted);
end
-- Initialize if no other mod has, or this version is newer than the previously initialized version
if (retVal.isInitialized == nil or retVal.isInitialized < versionNumber) then
	LuaEvents.PoliciesGrantBuildings_IsInitialized.Add(isInitialized);
	-- Initialize the code
	GameEvents.CityCaptureComplete.Add(PolicyBuildings_CityCaptured);
	GameEvents.PlayerCityFounded.Add(PolicyBuildings_CityFounded);
	GameEvents.PlayerAdoptPolicy.Add(PolicyBuildings_PolicyAdopted);
	GameEvents.PlayerAdoptPolicyBranch.Add(PolicyBuildings_PolicyBranchAdopted);
end